package internal

import "go.opentelemetry.io/contrib/instrumentation/google.golang.org/grpc/otelgrpc/internal"

Package internal provides internal functionality for the otelgrpc package.

Index

Functions

func ParseFullMethod

func ParseFullMethod(fullMethod string) (string, []attribute.KeyValue)

ParseFullMethod returns a span name following the OpenTelemetry semantic conventions as well as all applicable span attribute.KeyValue attributes based on a gRPC's FullMethod.

Parsing is consistent with grpc-go implementation: https://github.com/grpc/grpc-go/blob/v1.57.0/internal/grpcutil/method.go#L26-L39

func ParseFullMethodOld

func ParseFullMethodOld(fullMethod string) (string, []attribute.KeyValue)

ParseFullMethodOld returns a span name following the old OpenTelemetry semantic conventions as well as all applicable span attribute.KeyValue attributes based on a gRPC's FullMethod. Based on the implementation in: https://github.com/open-telemetry/opentelemetry-go-contrib/blob/072dcf8ad7e5e48b506e05720b29d8b078759606/instrumentation/google.golang.org/grpc/otelgrpc/internal/parse.go#L20

Source Files

parse.go

Directories

PathSynopsis
internal/testPackage test contains functions used by interop client/server.
Version
v0.68.0 (latest)
Published
Apr 7, 2026
Platform
linux/amd64
Imports
4 packages
Last checked
11 minutes ago

Tools for package owners.